#include "error.inc"

!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv!
!                                                                      !
!  Module name: USR_RATES                                              !
!                                                                      !
!  Purpose: Hook for user defined reaction rates.                      !
!                                                                      !
!  Author: J.Musser                                   Date: 10-Oct-12  !
!                                                                      !
!  Comments: Write reaction rates in units of moles/sec.cm^3 (cgs) or  !
!  kmoles/sec.m^3 (SI). Units should match those specified in the data !
!  file.
!                                                                      !
!  Example reaction: Methane combustion                                !
!                                                                      !
!  mfix.dat input:                                                     !
!``````````````````````````````````````````````````````````````````````!
!    @(RXNS)                                                           !
!      CH4_Comb { chem_eq = "CH4 + 2.0*O2 --> CO2 + 2.0*H2O" }         !
!    @(END)                                                            !
!``````````````````````````````````````````````````````````````````````!
!                                                                      !
!  usr_rates.f input:                                                  !
!``````````````````````````````````````````````````````````````````````!
!    c_O2  = (RO_g(IJK)*X_g(IJK,O2)/MW_g(O2))                          !
!    c_CH4 = (RO_g(IJK)*X_g(IJK,CH4)/MW_g(CH4))                        !
!    RATES(CH4_Comb) = 2.0d5 * EP_g(IJK) * c_O2 * c_CH4                !
!``````````````````````````````````````````````````````````````````````!
!  * Species alias and reaction names given in the data file can be    !
!    used in reference to the reaction index in RATES and a species    !
!    index in gas/solids phase variables.                              !
!                                                                      !
!  * Additional information is provided in section 5.11 of the code    !
!    Readme.                                                           !
!                                                                      !
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^!
      SUBROUTINE USR_RATES(IJK, RATES)

! Modules
!---------------------------------------------------------------------//
      use constant, only: C
      use fldvar, only: p_g, t_g, x_g, ro_g, ep_g
      use fldvar, only: t_s, x_s, ro_s, ep_s

! geometry for debugging
      use geometry, only: imax, jmax
      use param1, only: one, zero
      use param1, only: small_number, large_number
      use physprop, only: mu_g, mw_g, mw_mix_g
      use physprop, only: mw_s, nmax
      use physprop, only: dif_s

      use rxns, only: no_of_rxns, nrr, reactionrates
      use run, only: units
      use scales, only: unscale_pressure
      use toleranc, only: tmax, zero_x_gs
      use visc_s, only: mu_s

      use usr, only: index_liq, index_sol
      use usr, only: calc_mw_mix_l
      use usr, only: fwetarea_pack, sa_pack
      use usr, only: omega_l
! this routine is still being called here but is also being
! stored in the global variable for the liquid phase diffusivist
! therefore it can probably be removed
      use usr, only: calc_d_co2inmea

! these routines are called from rrates alone
      use usr, only: calc_d_meainmea
      !use usr, only: calc_h_co2inmea
      use usr, only: calc_k1_hikita, calc_k2_hikita, calc_k3_hikita, &
	  calc_k4_hikita, calc_k5_hikita, calc_k6_hikita       !chenboyan input
! Flag for chemistry scheme
      use usr, only: absorption_chem_type_enum
      use usr, only: single_step, equilibrium_segregated
      use usr, only: equilibrium_coupled
! Flag for mass transfer scheme
      use usr, only: mass_transfer_type, mass_transfer_type_enum
      use usr, only: onda_1968_mtc, psfirst_lmtc
      use usr, only: constant_mtc
! Function for equilibrium constants
      !use usr, only: keq_aboudheir   chenboyan 
      use error_manager

      use compar, only: myPe
      use exit, only: mfix_exit
      use funits, only: dmp_log, unit_log
      use machine, only: start_log, end_log
      use open_files_mod, only: open_pe_log

      IMPLICIT NONE
! Dummy arguments
!---------------------------------------------------------------------//
      INTEGER, INTENT(IN) :: IJK
      DOUBLE PRECISION, DIMENSION(NO_OF_RXNS), INTENT(OUT) :: RATES

! Local variables
!---------------------------------------------------------------------//
! local index for liquid and solids phases
      INTEGER :: iliquid
      INTEGER :: isolid

      INTEGER :: IER

! loop indice
      INTEGER :: L

! Species loop index
      INTEGER :: N

! absolute gas phase pressure (barye)
      DOUBLE PRECISION:: Pg_abs

! bounded phase temperatures (K)
      DOUBLE PRECISION :: xTg
      DOUBLE PRECISION :: xTl

! gas phase partial pressures (barye or dyne/cm^2)
      DOUBLE PRECISION :: pp_g(nmax(0))

! gas phase molar concentrations (gmol/cm^3)
      DOUBLE PRECISION :: c_g(nmax(0))

! gas phase reaction limiters: (0,1]
      DOUBLE PRECISION :: r_g(nmax(0))

! liquid phase molar concentration (gmol/cm^3)
! in units per liquid vol not cell volume!
      DOUBLE PRECISION :: c_l(nmax(index_liq))

! liquid phase mole fraction
      DOUBLE PRECISION :: y_l(nmax(index_liq))

! liquid phase reaction limiters: (0,1]
      DOUBLE PRECISION :: r_l(nmax(index_liq))

! liquid phase average molecular weight (g/gmol)
      DOUBLE PRECISION :: MW_MIX_L

! partial pressure of (gas) CO2 at interface (barye)
! concentration of (liq) CO2 at interface and (mol/cm^3)
      DOUBLE PRECISION :: p_CO2int
      DOUBLE PRECISION :: c_lCO2int

! CO2 partial pressure drop from bulk gas to interface (barye)
      DOUBLE PRECISION :: delta_pp_CO2

! CO2 concentration bump from interface to bulk (gmol/cm^3)
      DOUBLE PRECISION :: delta_ci_CO2

! load of CO2 in aq amine (gmol CO2/gmol amine)
      DOUBLE PRECISION :: load_CO2

! mass percentage of amine (%)
      DOUBLE PRECISION :: xp_lRNH2

! volume fraction of packing (1-bed porosity)
      DOUBLE PRECISION :: eps_pack

! volume fraction of liquid
      DOUBLE PRECISION :: eps_liq

! volume (void) fraction of gas
      DOUBLE PRECISION :: ep_gas

! local aliases for physical properties of gas
      DOUBLE PRECISION :: mu_gas
      DOUBLE PRECISION :: rho_gas

! physical properties of liquid
      DOUBLE PRECISION :: mu_liq
      DOUBLE PRECISION :: rho_liq

! diffusivsities (cm^2/s)
      DOUBLE PRECISION :: D_CO2inMEA
      DOUBLE PRECISION :: D_MEAinMEA
      DOUBLE PRECISION :: D_CO2inAIR

! henry's law constant (barye.cm^3/mol)
      DOUBLE PRECISION :: H_CO2inMEA

! surface tension of solution (dyne/cm)
      DOUBLE PRECISION :: omega_liq

! second order rate constant for CO2 reacting with MEA
      DOUBLE PRECISION :: kl_rxn, k2_rxn, k3_rxn, k4_rxn, k5_rxn, k6_rxn

! intermediate values for carabmate and bicarbonate formation calculation
      DOUBLE PRECISION :: k_carbf, k_bicarbf

! temporary reaction kinetics/rates
      DOUBLE PRECISION :: kfwd, krvs
      DOUBLE PRECISION :: fwd, rvs
      DOUBLE PRECISION :: keq

! gas phase mass transfer coefficient for CO2 (mol/(barye.cm^2.s))
! liquid phase mass transfer coefficient for CO2 (cm/s)
! overall mass transfer coefficient for CO2
      DOUBLE PRECISION :: kg_abs
      DOUBLE PRECISION :: kl_abs
      DOUBLE PRECISION :: kov_abs
! temporary calculations for overall mass transfer
      DOUBLE PRECISION :: gres, lres
! enhancement factor
      DOUBLE PRECISION :: E_Fac

! wetted and interfacial area of packing (cm^2/cm^3)
! in units per volume of bed
      DOUBLE PRECISION :: wa_pack
      DOUBLE PRECISION :: int_area

! Minimum species mass fraction required to facilitate a reaction.
      DOUBLE PRECISION, parameter :: x_Limiter = 1.0d-6
! Minimum phase volume fraction required to facilitate a reaction.
      DOUBLE PRECISION, parameter :: v_Limiter = 1.0d-6
! Minimum gradient at interface required to facilitate a flux.
      DOUBLE PRECISION, parameter :: p_Limiter = 1.0d-6
      double precision :: lp_limiter

! Reaction softener parameters.
      DOUBLE PRECISION :: rLimiter(NO_OF_RXNS)
      DOUBLE PRECISION, parameter :: r_pnt = 1.0d-4
      DOUBLE PRECISION, parameter :: r_exp = 3.0d0

!---------------------------------------------------------------------//
      INCLUDE 'species.inc'


! Constants that need to be set
!---------------------------------------------------------------------//
! solids phase index representing the liquid phase
! reaction occur between gas and liquid phase
      iliquid = index_liq

! solids phase index represent the solids (packing) phase
      isolid = index_sol

! Volume (void) fraction of gas phase
      ep_gas = EP_G(IJK)

! Volume fraction of liquid
      eps_liq = ep_s(ijk,iliquid)

! Volume fraction of packing
      eps_pack = ep_s(ijk,isolid)


! Gas phase quantities
!---------------------------------------------------------------------//
! initialize gas phase partial pressures and molar concentrations
      pp_g(:) = zero; c_g(:) = zero; r_g(:) = one;

! initialize
      Pg_abs = zero
      xTg = zero
      mu_gas = zero
      rho_gas = zero
      D_CO2inAIR = zero

      IF (ep_gas > v_limiter) THEN
! Make sure it is 'absolute pressure' (barye or Pa)
         Pg_abs = unscale_pressure(P_g(ijk))

! Bounded gas phase temperature (K)
         xTg   = min(TMAX,T_g(IJK))

! Local aliases
         mu_gas = MU_G(IJK)
         rho_gas = RO_G(IJK)

! diffusivsity of CO2 in Air
         D_CO2inAIR = 0.16   ! (cm^2/s)
         IF (UNITS=='SI') D_CO2inAIR=D_CO2inAIR/10000.d0 !(m^2/s)

! Compute the species concentrations and partial pressures:
! Note: concentration is in per gas volume NOT per cell volume
         DO N = 1, NMAX(0)
            IF (X_g(IJK,N) > x_limiter) THEN
               pp_g(N) = pg_abs*X_g(IJK,N)*&
                  (MW_MIX_g(IJK)/MW_g(N))             ! (barye or Pa)
               c_g(N) = ro_g(IJK)*X_g(IJK,N)/MW_g(N)  ! (gmol/cm^3 or kmol/m^3)
               r_g(N) = (X_g(IJK,N)/&
                  (X_g(IJK,N)+r_pnt))**r_exp
            ENDIF
         ENDDO
      ENDIF

! Liquid phase quantities
!---------------------------------------------------------------------//
! initialize liquid phase molar concentrations and mole fractions
      c_l(:) = zero; y_l(:) = zero; r_l(:) = one

! initialize
      xTl = zero
      MW_MIX_l = zero
      omega_liq = zero
      rho_liq = zero
      mu_liq = zero
      D_CO2inMEA = zero
      D_MEAinMEA = zero
      H_CO2inMEA = zero
      load_CO2 = zero
      xp_lRNH2 = zero

! If the fluid cell contains the liquid phase, calculate the properties
      IF (eps_liq > v_limiter) THEN

! Compute average molecular weight of liquid phase
         MW_MIX_L = calc_MW_mix_l(ijk, iliquid)

! Compute the species concentrations, etc:
! Note: concentration is in per liquid volume NOT per cell volume
         DO N = 1, NMAX(iliquid)
            IF (X_s(IJK,iliquid,N) > zero_x_gs) THEN !x_limiter) THEN
               c_l(N) = (ro_s(IJK,iliquid)*X_s(IJK,iliquid,N))/&
                  MW_s(iliquid,N)                     ! (gmol/cm^3)
               y_l(N) = X_s(IJK,iliquid,N)*&
                  (MW_MIX_L/MW_s(iliquid,N))           ! (mol-CO2/mol-Mix)
               r_l(N) = (X_s(IJK,iliquid,N)/&
                  (X_s(IJK,iliquid,N)+r_pnt))**r_exp
            ENDIF
         ENDDO

! bounded liquid phase temperature
         xTl = min(TMAX,T_s(IJK,iliquid))

! mass percentage of mea
         !xp_lRNH2 = X_S(IJK,iliquid,lRNH2)*100

! CO2 loading
         !IF (y_l(lRNH2) > ZERO) THEN
            !load_CO2 = y_l(lCO2)/y_l(lRNH2)
         !ELSE
            !load_CO2 = zero
         !ENDIF

! Some of the physical property correlations will need to be placed in
! their appropriate routines (viscosity, density)

! surface tension of the solution  (dyne/cm or N/m)
         omega_liq = omega_l(ijk)

! density of liquid phase (g/cm^3 or kg/m^3)
         rho_liq = ro_s(ijk,iliquid)

! viscosity of liquid phase (poise or Pa.s)
         mu_liq = mu_s(ijk,iliquid)

! diffusivity of CO2 in MEA solution (cm^2/s or m^2/s)
!         D_CO2inMEA = calc_D_CO2inMEA(xTl, mu_liq)
         !D_CO2inMEA = dif_s(ijk,iliquid,lCO2)     chenboyan

! diffusivity of MEA in MEA solution (cm^2/s or m^2/s)
         !D_MEAinMEA = calc_D_MEAinMEA(xTl, c_l(lCO2))

! henry's law constant for CO2 in solution (barye.cm^3/mol or
! Pa.m^3/kmol)
         !H_CO2inMEA = calc_H_CO2inMEA(xTl)    chenboyan

      ENDIF

! Packing phase and related quantities
!---------------------------------------------------------------------//
! initialize
      !kl_abs = zero
      !kg_abs = zero
      kl_rxn = zero
	  k2_rxn = zero
	  k3_rxn = zero
	  k4_rxn = zero
	  k5_rxn = zero
	  k6_rxn = zero
      wa_pack = zero
      int_area = zero

      IF (eps_pack > v_limiter .AND. eps_liq > v_limiter) THEN

! compute the wetted area (cm^2/cm^3 or m^2/m^3)
! Note: this is per unit volume of bed (not computational cell)
         wa_pack = sa_pack*fwetarea_pack(ijk)
! Compute the interfacial area: Assume the interfacial area equals the
! wetted area (length^2/length^3) which is per unit volume of bed NOT
! per unit volume of packing
         int_area = wa_pack
! second order rate constant for CO2 reacting with MEA (cm^3/s.gmol or
! m^3/s.kmol)
         !k1_rxn = calc_k1_hikita(xTl)   !chenboyan input
		 !k2_rxn = calc_k2_hikita(xTl)
		 !k3_rxn = calc_k3_hikita(xTl)
		 !k4_rxn = calc_k4_hikita(xTl)
		 !k5_rxn = calc_k5_hikita(xTl)
		 !k6_rxn = calc_k6_hikita(xTl)
		 
      ENDIF





! initialize rates
      RATES(:) = ZERO


!**********************************************************************!
!                                                                      !
!                   Heterogenous Reaction Rates                        !
!                                                                      !
!**********************************************************************!
! Reactions are suppressed if both phases are not present

! baoheting --> g baoheting
!---------------------------------------------------------------------//

         kl_rxn = calc_k1_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(baoheting)*kl_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_1) = FWD

!**********************************************************************!
!                                                                      !
!                Homogenous liquid phase reactions                     !
!                                                                      !
!**********************************************************************!
! Liquid phase reactions are suppressed if liquid phase is not present
! through calculation of concentrations (see earlier).
! Multiple by liquid phase reactions by ep_l to account for the fact
! that only part of the volume is occupied by liquid


      SELECT CASE(ABSORPTION_CHEM_TYPE_ENUM)

      CASE (SINGLE_STEP)

! Name (rxn ID): CARBAMATE_FORM_SINGLE_STEP
! In accommodating this rxn, the mass transfer step above must be
! evaluated as physical mass transfer only (i.e. no enhancement factor)

! second order rate constant for CO2 reacting with MEA (cm^3/s.gmol or
! m^3/s.kmol)
         

 ! Single step carbamate formation:  fangxiangting --> baoheting
!---------------------------------------------------------------------//

         k2_rxn = calc_k2_hikita(xTl)                            !chenboyan input
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(fangxiangting)*k2_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_2) = FWD
		 
! Single step carbamate formation:  jiaozhi --> fangxiangting
!---------------------------------------------------------------------//

         k3_rxn = calc_k3_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(jiaozhi)*k3_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_3) = FWD
		 
! Single step carbamate formation:  jiaozhi --> liqingzhi
!---------------------------------------------------------------------//

         k4_rxn = calc_k4_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(jiaozhi)*k4_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_4) = FWD
		 
! Single step carbamate formation:  jiaozhi --> zhongjianxiang
!---------------------------------------------------------------------//

         k4_rxn = calc_k4_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(jiaozhi)*k4_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_5) = FWD
		 
! Single step carbamate formation:  liqingzhi --> jiaozhi
!---------------------------------------------------------------------//

         k5_rxn = calc_k5_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(liqingzhi)*k5_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_6) = FWD
		 
! Single step carbamate formation:  zhongjianxiang --> jiaozhi
!---------------------------------------------------------------------//

         k5_rxn = calc_k5_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(zhongjianxiang)*k5_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_7) = FWD
		 
! Single step carbamate formation:  zhongjianxiang --> coke
!---------------------------------------------------------------------//

         k6_rxn = calc_k6_hikita(xTl)
         !K_CARBF = eps_liq*kl_rxn
         FWD = c_l(zhongjianxiang)*k6_rxn
         !FWD = K_CARBF*FWD
         RATES(Reaction_8) = FWD
		 
      CASE (EQUILIBRIUM_COUPLED)
! do nothing here as the equilibrium reactions are handled    在平衡反应分开处理的时候什么都不做  
! separately
      CASE DEFAULT
      END SELECT
!**********************************************************************!
!                     Setup/Apply Limiter                              !
!**********************************************************************!

      rLIMITER = ONE
      lp_limiter = p_limiter  !(barye)

      IF (UNITS=='SI') lp_limiter = p_limiter/10.d0   !(pa)

      !IF (delta_pp_co2 > lp_limiter) THEN
      !   rLIMITER(CO2_absorption) = (delta_pp_CO2/(delta_pp_CO2+r_pnt))**r_exp         !chenboyan
      !ENDIF

      !rLIMITER(CO2_absorption) = r_g(gCO2)

      !rLIMITER(CARBAMATE_FORM_SINGLE_STEP) = min(r_l(lCO2),r_l(lRNH2))

      !rLIMITER(FWD_REVERSION_CARBAMATE) = min(r_l(RNHCO2m), r_l(lH2O))
      !rLIMITER(REV_REVERSION_CARBAMATE) = min(r_l(lRNH2), r_l(HCO3m))

      !rLIMITER(FWD_DISSOCIATION_CO2) = min(r_l(lCO2),r_l(lH2O))
      !rLIMITER(REV_DISSOCIATION_CO2) = min(r_l(HCO3m),r_l(H3Op))

      !rLIMITER(FWD_DISSOCIATION_BICARBONATE) = min(r_l(HCO3m),r_l(lH2O))
      !rLIMITER(REV_DISSOCIATION_BICARBONATE) = min(r_l(CO3m2),r_l(H3Op))

      !rLIMITER(FWD_DISSOCIATION_MEAH) = min(r_l(RNH3p),r_l(lH2O))
      !rLIMITER(REV_DISSOCIATION_MEAH) = min(r_l(lRNH2),r_l(H3Op))

      !rLIMITER(FWD_IONIZATION_H2O) = r_l(lH2O)
      !rLIMITER(REV_IONIZATION_H2O) = min(r_l(OHm),r_l(H3Op))
	  rLIMITER(Reaction_1) = r_l(baoheting)
	  rLIMITER(Reaction_2) = r_l(fangxiangting)
	  rLIMITER(Reaction_3) = r_l(jiaozhi)
	  rLIMITER(Reaction_4) = r_l(jiaozhi)
	  rLIMITER(Reaction_5) = r_l(jiaozhi)
	  rLIMITER(Reaction_6) = r_l(liqingzhi)
	  rLIMITER(Reaction_7) = r_l(zhongjianxiang)
	  rLIMITER(Reaction_8) = r_l(zhongjianxiang)

      DO L=1,2 !NO_OF_RXNS !(up to equilbrium reactions)
         RATES(L) = RATES(L) * rLimiter(L)
      ENDDO


!**********************************************************************!
!                         Save the Rates                               !
!**********************************************************************!
      DO L=1, min(nRR,NO_OF_RXNS)
         ReactionRates(IJK,L) = RATES(L)
      ENDDO

      RETURN

      END SUBROUTINE USR_RATES


!----------------------------------------------------------------------!
! Subroutine: calc_kg_onda                                             !
!                                                                      !
! Purpose: Calculate the gas phase mass transport coefficient for      !
! absorption of pure CO2 into an organic solvent in a packed bed.      !
!                                                                      !
! This reference studies CO2 absorbed into methanol and carbon         !
! tetracholoride. The column is random packed bed (raschig rings,      !
! berl saddles, spheres, etc). The mass transfer coefficient is        !
! returned in units (mol/(barye.cm^2.s) or kmol/(Pa.m^2.s)). This is   !
! considered mass transfer due to physical absorption only (no rxn).   !
!                                                                      !
! References:                                                          !
! Onda K. et al., Mass transfer coefficients between gas and liquid    !
!   phases in packed columns, Journal of Chemical Engineering of       !
!   Japan, 1968, 56-62.                                                !
!                                                                      !
!''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''!
 

!----------------------------------------------------------------------!
! Subroutine: calc_kl_onda                                             !
!                                                                      !
! Purpose: Calculate the liquid phase mass transport coefficient for   !
! absorption of pure CO2 into an organic solvent in a packed bed.      !
!                                                                      !
! This reference studies CO2 absorbed into methanol and carbon         !
! tetracholoride. The column is random packed bed (raschig rings,      !
! berl saddles, spheres, etc). The mass transfer coefficient is        !
! returned in units (cm/s or m/s). This is considered mass transfer    !
! due to physical absorption only (no rxn).                            !
!                                                                      !
! References:                                                          !
! Onda K. et al., Mass transfer coefficients between gas and liquid    !
!   phases in packed columns, Journal of Chemical Engineering of       !
!   Japan, 1968, 56-62.                                                !
!                                                                      !
!''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''!
      




!----------------------------------------------------------------------!
! Subroutine: calc_lCO2int                                             !
!                                                                      !
! Purpose: Calculate the concentration of CO2 in the liquid at the     !
! interface. The concentration of CO2 is returned in units g-mol/cm^3  !
! in CGS or kmol/m^3 in SI.                                            !
!                                                                      !
! References:                                                          !
! Pandya, J. D., Chemical Engineering Communications, Vol. 19, 1983,   !
!   343-361.                                                           !
!   equations 9, 12, 33                                                !
!                                                                      !
!''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''!
     

! 

!----------------------------------------------------------------------!
! Function: calc_E_wellek                                              !
!                                                                      !
! Purpose: Calculate the enhancement factor for second order reaction  !
! reaction which represents the ratio of mass transfer coefficient for !
! absorption when chemical reaction occurs to pure physical absorption !
! without chemical reaction. Dimensionless.                            !
!                                                                      !
! References:                                                          !
! Wellek, R. M., et al., the Canadian Journal of Chemical Engineering  !
!   56, 1978, 181-186                                                  !
!   equations 18a, 18b, 5, 4                                           !
!                                                                      !
!''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''!
 


!----------------------------------------------------------------------!
! Function: calc_PCO2int                                               !
!                                                                      !
! Purpose: Calculate the partial pressure of CO2 in the gas phase      !
! at the interface. A number of assumptions are implicit in this       !
! model. Namely, the reaction is assumed to be fast enough for the     !
! reaction to occur primarily in the film. Therefore the concentration !
! of dissolved gas (CO2) in the bulk, and other species, are very near !
! in equilbrium with bulk of liquid. The partial pressure of CO2 is    !
! returned in units barye or Pa.                                       !
!                                                                      !
! References:                                                          !
! Pandya, J. D., Chemical Engineering Communications, Vol. 19, 1983,   !
!   343-361.                                                           !
!   equations 9, 12, 33                                                !
!                                                                      !
!''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''!


    